@@ -198,6 +198,8 @@ DialogImport::DialogImport( BaseObjectType* cobject,
                             const Glib::RefPtr< Gtk::Builder >& refbuilder )
 :   DialogEvent( cobject, refbuilder ), m_dialog_password( nullptr ), m_diary( nullptr )
 {
+    refbuilder->get_widget( "B_import_import", m_B_import );
+    refbuilder->get_widget( "B_import_cancel", m_B_cancel );
     refbuilder->get_widget( "FB_import_diary", m_file_chooser_button );
     refbuilder->get_widget( "L_import_info", m_label_info );
     refbuilder->get_widget( "L_import_entries", m_label_entries );
@@ -225,6 +227,11 @@ DialogImport::DialogImport( BaseObjectType* cobject,
             sigc::mem_fun( this, &DialogImport::handle_parts_changed ) );
     m_checkbutton_import_chapters->signal_toggled().connect(
             sigc::mem_fun( this, &DialogImport::handle_parts_changed ) );
+
+    m_B_import->signal_clicked().connect(
+            sigc::bind( sigc::mem_fun( this, &DialogEvent::response ), RESPONSE_GO ));
+    m_B_cancel->signal_clicked().connect(
+            sigc::bind( sigc::mem_fun( this, &DialogEvent::response ), RESPONSE_CANCEL ));
 }
 
 inline bool
@@ -301,15 +308,13 @@ DialogImport::handle_parts_changed()
     m_option_import_chapters = m_checkbutton_import_chapters->get_active();
 
     m_tag_widget->set_sensitive( m_option_import_entries );
-
-    set_response_sensitive( RESPONSE_GO,
-            m_flag_diary_is_ready && is_options_ready() );
+    m_B_import->set_sensitive( m_flag_diary_is_ready && is_options_ready() );
 }
 
 void
 DialogImport::set_ready()
 {
-    set_response_sensitive( RESPONSE_GO, is_options_ready() );
+    m_B_import->set_sensitive( is_options_ready() );
 
     int n_chapters = 0;
     for( auto& kv_ctg : m_diary->m_chapter_categories )
@@ -331,7 +336,7 @@ DialogImport::set_ready()
 void
 DialogImport::set_not_ready( const Glib::ustring& message )
 {
-    set_response_sensitive( RESPONSE_GO, false );
+    m_B_import->set_sensitive( false );
 
     m_label_entries->set_text( "-" );
     m_label_tags->set_text( "-" );
