@@ -199,7 +199,7 @@ Diary::set_path( const std::string& path0, SetPathType type )
 #endif
 
     // CHECK FILE SYSTEM PERMISSIONS
-    if( access( path.c_str(), F_OK ) != 0 ) // check existence
+    if( access( PATH( path ).c_str(), F_OK ) != 0 ) // check existence
     {
         if( errno == ENOENT )
         {
@@ -212,12 +212,12 @@ Diary::set_path( const std::string& path0, SetPathType type )
         else
             return LIFEO::FAILURE; // should not be the case
     }
-    else if( access( path.c_str(), R_OK ) != 0 ) // check read access
+    else if( access( PATH( path ).c_str(), R_OK ) != 0 ) // check read access
     {
         PRINT_DEBUG( "File is not readable" );
         return LIFEO::FILE_NOT_READABLE;
     }
-    else if( type != SPT_READ_ONLY && access( path.c_str(), W_OK ) != 0 ) // check write access
+    else if( type != SPT_READ_ONLY && access( PATH( path ).c_str(), W_OK ) != 0 ) // check write access
     {
         if( type == SPT_NEW )
         {
@@ -233,7 +233,7 @@ Diary::set_path( const std::string& path0, SetPathType type )
     if( type != SPT_READ_ONLY )
     {
         std::string path_lock( path + LOCK_SUFFIX );
-        if( access( path_lock.c_str(), F_OK ) == 0 )
+        if( access( PATH( path_lock ).c_str(), F_OK ) == 0 )
         {
             if( s_flag_ignore_locks )
                 print_info( "Ignored file lock" );
@@ -243,7 +243,7 @@ Diary::set_path( const std::string& path0, SetPathType type )
 
         if( type == SPT_NORMAL )
         {
-            FILE* fp = fopen( path_lock.c_str(), "a+" );
+            FILE* fp = fopen( PATH( path_lock ).c_str(), "a+" );
             if( fp )
                 fclose( fp );
             else
@@ -324,7 +324,7 @@ Diary::close_file()
 LIFEO::Result
 Diary::read_header()
 {
-    m_ifstream = new std::ifstream( m_path.c_str() );
+    m_ifstream = new std::ifstream( PATH( m_path ).c_str() );
 
     if( ! m_ifstream->is_open() )
     {
@@ -416,7 +416,7 @@ Diary::read_encrypted()
 
 #ifdef _WIN32
     m_ifstream->close();
-    m_ifstream->open( m_path.c_str(),
+    m_ifstream->open( PATH( m_path ).c_str(),
                       std::ifstream::in | std::ifstream::binary | std::ifstream::ate );
 
     size_t fsize = m_ifstream->tellg();
@@ -1446,7 +1446,7 @@ create_db_tag_text( const Tag* tag, std::stringstream& output )
     output << tag->get_chart_type() << '\n';
 
     if( ! tag->is_boolean() && ! tag->get_unit().empty() ) // never true for untagged
-        output << "tu" << tag->get_unit() << '\n';
+        output << "tu" << std::string( tag->get_unit() ) << '\n';
 
     if( tag->get_has_own_theme() )
     {
@@ -1638,7 +1638,7 @@ Diary::create_db_body_text( std::stringstream& output )
         for( Tag* tag : entry->m_tags )
         {
             output << "T" << ( tag == entry->get_theme_tag() ? 'T' : '_' )
-                   << tag->get_name_and_value( entry, true, false ) << '\n';
+                   << std::string( tag->get_name_and_value( entry, true, false ) ) << '\n';
         }
 
         // LOCATION
@@ -1694,10 +1694,10 @@ Diary::write()
         copy_file_suffix( m_path, ".", m_read_version );
 
     // BACKUP THE PREVIOUS VERSION
-    if( access( m_path.c_str(), F_OK ) == 0 )
+    if( access( PATH( m_path ).c_str(), F_OK ) == 0 )
     {
         std::string path_old( m_path + ".~previousversion~" );
-        rename( m_path.c_str(), path_old.c_str() );
+        rename( PATH( m_path ).c_str(), PATH( path_old ).c_str() );
     }
 
     // WRITE THE FILE
@@ -1747,7 +1747,7 @@ Diary::write_copy( const std::string& path, const std::string& passphrase, bool
 LIFEO::Result
 Diary::write_txt( const std::string& path, bool flag_filtered_only )
 {
-    std::ofstream file( path.c_str(), std::ios::out | std::ios::trunc );
+    std::ofstream file( PATH( path ).c_str(), std::ios::out | std::ios::trunc );
     if( ! file.is_open() )
     {
         print_error( "i/o error: ", path );
@@ -1857,7 +1857,7 @@ LIFEO::Result
 Diary::write_plain( const std::string& path, bool flag_header_only )
 {
     // NOTE: ios::binary prevents windows to use \r\n for line ends
-    std::ofstream file( path.c_str(), std::ios::binary | std::ios::out | std::ios::trunc );
+    std::ofstream file( PATH( path ).c_str(), std::ios::binary | std::ios::out | std::ios::trunc );
     if( ! file.is_open() )
     {
         print_error( "i/o error: ", path );
@@ -1883,7 +1883,7 @@ Diary::write_encrypted( const std::string& path )
 {
     // writing header:
     write_plain( path, true );
-    std::ofstream file( path.c_str(), std::ios::out | std::ios::app | std::ios::binary );
+    std::ofstream file( PATH( path ).c_str(), std::ios::out | std::ios::app | std::ios::binary );
     if( ! file.is_open() )
     {
         print_error( "i/o error: ", path );
@@ -2014,8 +2014,8 @@ Diary::remove_lock()
         return false;
 
     std::string path_lock( m_path + LOCK_SUFFIX );
-    if( access( path_lock.c_str(), F_OK ) == 0 )
-        remove( path_lock.c_str() );
+    if( access( PATH( path_lock ).c_str(), F_OK ) == 0 )
+        remove( PATH( path_lock ).c_str() );
     return true;
 }
 
