@@ -469,7 +469,8 @@ Tag::get_combined_value() const
     for( auto& iter : *this )
         grand += iter.second;
 
-    return( ( m_chart_type & ChartPoints::AVERAGE ) != 0 ? grand/size() : grand );
+    return( ( m_chart_type & ChartPoints::AVERAGE ) == ChartPoints::AVERAGE ?
+                    grand/size() : grand );
 }
 
 ChartPoints*
