@@ -490,7 +490,7 @@ class ChartPoints
         static constexpr int        AVERAGE             = 0x30;
         static constexpr int        VALUE_TYPE_MASK     = 0xf0;
 
-        static constexpr int        UNDERLAY_PREV_MONTH = 0x100;
+        static constexpr int        UNDERLAY_PREV_YEAR = 0x100;
 
         unsigned int                calculate_distance( const Date&, const Date& ) const;
         int                         calculate_distance_neg( const Date&, const Date& ) const;
@@ -529,16 +529,18 @@ class DiaryElementChart
         { return m_chart_type; }
         void                    set_chart_type( int type )
         {
-            if( type == ChartPoints::UNDERLAY_PREV_MONTH )
-                m_chart_type |= ChartPoints::UNDERLAY_PREV_MONTH;
-            else if( type == -ChartPoints::UNDERLAY_PREV_MONTH )
-                m_chart_type &= (~ChartPoints::UNDERLAY_PREV_MONTH);
+            if( type == ChartPoints::UNDERLAY_PREV_YEAR )
+                m_chart_type |= ChartPoints::UNDERLAY_PREV_YEAR;
+            else if( type == -ChartPoints::UNDERLAY_PREV_YEAR )
+                m_chart_type &= (~ChartPoints::UNDERLAY_PREV_YEAR);
             else
             {
                 if( ( type & ChartPoints::PERIOD_MASK ) == 0 )
                     type |= ( m_chart_type & ChartPoints::PERIOD_MASK );
                 if( ( type & ChartPoints::VALUE_TYPE_MASK ) == 0 )
                     type |= ( m_chart_type & ChartPoints::VALUE_TYPE_MASK );
+                if( ( type & ChartPoints::UNDERLAY_PREV_YEAR ) == 0 )
+                    type |= ( m_chart_type & ChartPoints::UNDERLAY_PREV_YEAR );
                 m_chart_type = type;
             }
         }
