@@ -312,6 +312,23 @@ Date::get_weekday() const
     return( d % 7 );
 }
 
+Ustring
+Date::get_weekday_str() const
+{
+    struct tm ti;
+    ti.tm_wday = get_weekday();
+
+#ifdef _WIN32
+    wchar_t buf[ 32 ];
+    wcsftime( buf, 32, L"%A", &ti );
+    return Ustring( convert_utf16_to_8( buf ) );
+#else
+    char buf[ 32 ];
+    strftime( buf, 32, "%A", &ti );
+    return Ustring( buf );
+#endif
+}
+
 unsigned int
 Date::get_days_in_month() const
 {
@@ -478,7 +495,7 @@ midtone( COLORREF c1, COLORREF c2, float ratio )
             ( int ) ( ( GetGValue( c1 ) * ratio ) + ( GetGValue( c2 ) * ( 1.0 - ratio ) ) ),
             ( int ) ( ( GetBValue( c1 ) * ratio ) + ( GetBValue( c2 ) * ( 1.0 - ratio ) ) ) );
 }
-    
+
 #endif
 
 // FILE OPERATIONS =================================================================================
@@ -522,7 +539,7 @@ copy_file_suffix( const std::string& source_path, const std::string& suffix1, in
                 ss << suffix2;
             int index( 0 );
 
-            while( access( ss.str().c_str(), F_OK ) == 0 )
+            while( access( PATH( ss.str() ).c_str(), F_OK ) == 0 )
             {
                 ss.str( std::string() );
                 ss.clear();
@@ -550,11 +567,11 @@ copy_file_suffix( const std::string& source_path, const std::string& suffix1, in
 }
 
 bool
-is_dir( const char* path )
+is_dir( const std::string& path )
 {
     struct stat s;
 
-    if( stat( path, &s ) == 0 )
+    if( stat( PATH( path ).c_str(), &s ) == 0 )
     {
         if( s.st_mode & S_IFDIR )
             return true;
@@ -567,6 +584,17 @@ is_dir( const char* path )
         throw Error( "Stat failed" );
 }
 
+#ifdef _WIN32
+std::string
+convert_filename_to_win32_locale( const std::string& fn )
+{
+    char* buf = g_win32_locale_filename_from_utf8( fn.c_str() );
+    std::string path{ buf };
+    delete buf;
+    return path;
+}
+#endif
+
 // TEXT OPERATIONS =================================================================================
 long
 convert_string( const std::string& str )
@@ -669,26 +697,6 @@ is_char_alpha( gunichar c )
 }
 
 #else
-wchar_t*
-convert_utf8_to_16( const Ustring& str8 )
-{
-    wchar_t* str16 = new wchar_t[ str8.size() + 1 ];
-    MultiByteToWideChar( CP_UTF8, 0, str8.c_str(), str8.size() + 1, str16, str8.size() + 1 );
-    
-    return str16;
-}
-
-char*
-convert_utf16_to_8( const wchar_t* str16 )
-{
-    char* str8{ nullptr };
-    int size = WideCharToMultiByte( CP_UTF8, 0, str16, -1, str8, 0, NULL, NULL );
-    str8 = new char[ size ];
-    WideCharToMultiByte( CP_UTF8, 0, str16, -1, str8, size, NULL, NULL );
-
-    return str8;
-}
-
 wchar_t
 char_lower( wchar_t c )
 {
@@ -784,6 +792,30 @@ command_line_to_argvA( PCHAR CmdLine, int* _argc )
 }
 #endif
 
+#ifdef _WIN32
+
+wchar_t*
+convert_utf8_to_16( const Ustring& str8 )
+{
+    wchar_t* str16 = new wchar_t[ str8.size() + 1 ];
+    MultiByteToWideChar( CP_UTF8, 0, str8.c_str(), str8.size() + 1, str16, str8.size() + 1 );
+
+    return str16;
+}
+
+char*
+convert_utf16_to_8( const wchar_t* str16 )
+{
+    char* str8{ nullptr };
+    int size = WideCharToMultiByte( CP_UTF8, 0, str16, -1, str8, 0, NULL, NULL );
+    str8 = new char[ size ];
+    WideCharToMultiByte( CP_UTF8, 0, str16, -1, str8, size, NULL, NULL );
+
+    return str8;
+}
+
+#endif
+
 // ENCRYPTION ======================================================================================
 bool
 Cipher::init()
@@ -808,7 +840,7 @@ Cipher::init()
     // allocate a pool of 16k secure memory. this makes the secure memory...
     // ...available and also drops privileges where needed
     gcry_control( GCRYCTL_INIT_SECMEM, 16384, 0 );
-     
+
     // resume warnings
     gcry_control( GCRYCTL_RESUME_SECMEM_WARN );
     */
