@@ -233,14 +233,7 @@ class Date
         std::string                 get_year_str() const;
         Ustring                     get_month_str() const;
         //std::string                 get_day_str() const;
-        Ustring                     get_weekday_str() const
-        {
-            struct tm ti;
-            ti.tm_wday = get_weekday();
-            char buf[ 32 ];
-            strftime( buf, 32, "%A", &ti );
-            return Ustring( buf );
-        }
+        Ustring                     get_weekday_str() const;
 
         unsigned int                get_days_in_month() const;
 
@@ -558,7 +551,7 @@ std::ios::pos_type  get_file_size( std::ifstream& );
 
 bool                copy_file_suffix( const std::string&, const std::string&, int );
 
-bool                is_dir( const char* );
+bool                is_dir( const std::string& );
 
 inline std::string
 get_filename_base( const std::string& path )
@@ -573,6 +566,16 @@ get_filename_base( const std::string& path )
 #endif
 }
 
+#ifdef _WIN32
+
+std::string
+convert_filename_to_win32_locale( const std::string& );
+
+#define PATH( A ) convert_filename_to_win32_locale( A )
+#else
+#define PATH( A ) ( A )
+#endif
+
 typedef std::set< std::string > ListPaths;
 
 // TEXT OPERATIONS =================================================================================
@@ -587,15 +590,17 @@ bool                is_char_alpha( gunichar );
 std::string         get_env_lang();
 
 #else
-wchar_t*            convert_utf8_to_16( const Ustring& );
-char*               convert_utf16_to_8( const wchar_t* );
-
 wchar_t             char_lower( wchar_t );
 bool                is_char_alpha( wchar_t );
 
 PCHAR*              command_line_to_argvA( PCHAR, int* );
 #endif
 
+#ifdef _WIN32
+wchar_t*            convert_utf8_to_16( const Ustring& );
+char*               convert_utf16_to_8( const wchar_t* );
+#endif
+
 // ENCRYPTION ======================================================================================
 class Cipher
 {
