@@ -34,7 +34,7 @@
 #include "view_entry.hpp"
 
 
-#define LIFEOGRAPH_BETA_RELEASE
+//#define LIFEOGRAPH_BETA_RELEASE
 
 #define LIFEOGRAPH_RELEASE_CODENAME "artemisia"
 
