@@ -408,8 +408,13 @@ EntryView::toggle_favoredness()
 void
 set_entry_todo_status( Entry* e, ElemStatus status )
 {
-    e->set_todo_status( status );
-    e->set_date_status( time( nullptr ) );
+    if( status == ES::NOT_TODO )
+        e->calculate_todo_status();
+    else
+    {
+        e->set_todo_status( status );
+        e->set_date_status( time( nullptr ) );
+    }
     AppWindow::p->panel_diary->handle_elem_changed( e );
 }
 void
@@ -425,11 +430,7 @@ EntryView::set_todo_status( ElemStatus status )
         AppWindow::p->panel_diary->do_for_each_selected_entry( s );
     }
     else
-    {
-        m_ptr2elem->set_todo_status( status );
-        m_ptr2elem->set_date_status( time( nullptr ) );
-        AppWindow::p->panel_diary->handle_elem_changed( m_ptr2elem );
-    }
+        set_entry_todo_status( m_ptr2elem, status );
 
     AppWindow::p->panel_main->set_icon( m_ptr2elem->get_icon32() );
     update_status_date();
