@@ -220,7 +220,7 @@ ViewLogin::populate_diaries()
         row[ colrec->name ] = Glib::filename_display_basename( *iter );
         row[ colrec->path ] = *iter;
 
-        if( stat( iter->c_str(), &fst ) == 0 )
+        if( stat( PATH( *iter ).c_str(), &fst ) == 0 )
         {
             time_t file_time( m_sort_date_type == SDT_SAVE ? fst.st_mtime : fst.st_atime );
             row[ colrec->date ] = Date::format_string_dt( file_time );
@@ -538,7 +538,8 @@ ViewLogin::show_info( Gtk::MessageType type, const Glib::ustring& text,
     m_B_info->set_label( button_text );
     m_resp_cur = response;
     m_IB_login->set_message_type( type );
-    m_G_login->attach( *m_IB_login, 0, 0, 1, 1 );
+    if( m_IB_login->get_parent() == nullptr )
+        m_G_login->attach( *m_IB_login, 0, 0, 1, 1 );
     m_IB_login->show();
     m_flag_info_is_visible = true;
 }
