@@ -55,7 +55,7 @@ ElementViewWithChart< T >::ElementViewWithChart( const Ustring type )
         throw LIFEO::Error( "Failed to create the chart" );
     }
 
-    m_A_underlay = Lifeograph::p->add_action_bool( "underlay_prev_month_" + type,
+    m_A_underlay = Lifeograph::p->add_action_bool( "underlay_prev_year_" + type,
             sigc::mem_fun( *this, &ElementViewWithChart::handle_chart_underlay ), false);
     m_Sc_chart->signal_change_value().connect(
             sigc::mem_fun( this, &ElementViewWithChart::handle_zoom ) );
@@ -132,7 +132,7 @@ ElementViewWithChart< T >::handle_chart_shown()
     }
 
     bool flag_underlay{ ElementView< T >::m_ptr2elem->get_chart_type() &
-                        ChartPoints::UNDERLAY_PREV_MONTH };
+                        ChartPoints::UNDERLAY_PREV_YEAR };
 
     m_A_underlay->change_state( flag_underlay );
 
@@ -151,9 +151,9 @@ ElementViewWithChart< T >::handle_chart_underlay()
     m_A_underlay->change_state( flag_underlay );
 
     if( flag_underlay )
-        ElementView< T >::m_ptr2elem->set_chart_type( ChartPoints::UNDERLAY_PREV_MONTH );
+        ElementView< T >::m_ptr2elem->set_chart_type( ChartPoints::UNDERLAY_PREV_YEAR );
     else
-        ElementView< T >::m_ptr2elem->set_chart_type( -ChartPoints::UNDERLAY_PREV_MONTH );
+        ElementView< T >::m_ptr2elem->set_chart_type( -ChartPoints::UNDERLAY_PREV_YEAR );
 
     update_points();
 }
@@ -1718,21 +1718,18 @@ DiaryView::start_dialog_import()
 {
     AppWindow::p->panel_main->hide_popover();
 
-    Glib::RefPtr< Gtk::Builder > builder(
-            Gtk::Builder::create_from_file( Lifeograph::UIDIR + "/import.ui" ) );
+    if( m_dialog_import == nullptr )
+        Lifeograph::get_builder2()->get_widget_derived( "D_import",
+                                                        m_dialog_import );
 
-    DialogImport* dialog_import;
-    builder->get_widget_derived( "D_import", dialog_import );
-
-    if(  dialog_import->run() == RESPONSE_GO )
+    if(  m_dialog_import->run() == RESPONSE_GO )
     {
         AppWindow::p->panel_diary->update_entry_list();
         AppWindow::p->panel_extra->populate();
         update_combobox_chapter_ctg();
     }
 
-    dialog_import->hide();
-    delete dialog_import;
+    m_dialog_import->hide();
 }
 
 void
@@ -1744,8 +1741,9 @@ DiaryView::start_dialog_export()
             DialogPassword::launch( DialogPassword::OT_AUTHENTICATE,
                                     m_ptr2elem ) == RESPONSE_GO : true )
     {
-        if( m_dialog_export == NULL )
-            Lifeograph::get_builder2()->get_widget_derived( "D_export", m_dialog_export );
+        if( m_dialog_export == nullptr )
+            Lifeograph::get_builder2()->get_widget_derived( "D_export",
+                                                            m_dialog_export );
 
         m_dialog_export->run();
         m_dialog_export->hide();
