@@ -430,7 +430,7 @@ WidgetChart::on_draw( const Cairo::RefPtr< Cairo::Context >& cr )
     cr->stroke();
 
     // GRAPH LINE PREVIOUS
-    if( period == ChartPoints::MONTHLY && ( m_points->type & ChartPoints::UNDERLAY_PREV_MONTH ) )
+    if( period == ChartPoints::MONTHLY && ( m_points->type & ChartPoints::UNDERLAY_PREV_YEAR ) )
     {
         const static std::valarray< double > dash_pattern{ 3.0 };
         cr->set_source_rgb( 1.0, 0.5, 0.5 );
