@@ -414,7 +414,7 @@ WidgetEntryList::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context
         return false;
 
     bool retval( false );
-    Gdk::DragAction action{ Gdk::ACTION_MOVE };
+    Gdk::DragAction action{ m_flag_text_drop ? Gdk::ACTION_COPY : Gdk::ACTION_MOVE };
     Gtk::TreePath path;
     DiaryElement* element{ nullptr };
 
