@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -63,6 +63,23 @@ TextbufferDiary::get_text_width( const Pango::FontDescription& fd, const Glib::u
     return width;
 }
 
+// UNDOEDIT
+void
+UndoEdit::insert()
+{
+    auto textbuffer = dynamic_cast< TextbufferDiary* >( m_ptr2buffer );
+
+    textbuffer->m_flag_ongoing_operation = true;
+    Gtk::TextIter iter_bgn = m_ptr2buffer->get_iter_at_offset( m_position );
+    Gtk::TextIter iter_end = m_ptr2buffer->insert( iter_bgn, m_text );
+
+    // handle reparsing internally
+    iter_bgn = m_ptr2buffer->get_iter_at_offset( m_position );
+    textbuffer->calculate_para_bounds( iter_bgn, iter_end );
+    textbuffer->parse( iter_bgn.get_offset(), iter_end.get_offset() );
+
+    textbuffer->m_flag_ongoing_operation = false;
+}
 
 // LINK
 Link::Link( const Glib::RefPtr< Gtk::TextMark >& start,
@@ -261,18 +278,7 @@ LinkCheck::set_check( int new_index )
     iter_start.backward_find_char( TextbufferDiary::s_predicate_nl );
     iter_end.forward_to_line_end();
 
-    buffer->parse( iter_start, iter_end );
-}
-
-InlineImage::InlineImage( int offset )
-:   m_offset( offset )
-{ }
-
-InlineImage::~InlineImage()
-{
-    /*Glib::RefPtr< Gtk::TextBuffer > buffer = m_mark->get_buffer();
-    if( buffer )
-        buffer->delete_mark( m_mark );*/
+    buffer->parse( iter_start.get_offset(), iter_end.get_offset() );
 }
 
 // TEXTBUFFERDIARY =================================================================================
@@ -301,12 +307,8 @@ TextbufferDiary::TextbufferDiary()
     m_tags.push_back( m_tag_match );
 
     m_tag_markup = Tag::create( "markup" );
-    //m_tag_markup->property_scale() = 0.8;
     m_tags.push_back( m_tag_markup );
 
-    m_tag_markup_link = Tag::create( "markup.link" );
-    m_tags.push_back( m_tag_markup_link );
-
     m_tag_hidden = Tag::create( "hidden" );
     m_tag_hidden->property_invisible() = true;
     m_tags.push_back( m_tag_hidden );
@@ -392,6 +394,8 @@ TextbufferDiary::handle_login()
     // SPELL CHECKING
     if( Diary::d->is_read_only() )
         m_flag_handle_word = false;
+
+    m_tag_markup->property_invisible() = Diary::d->is_read_only();
 }
 
 void
@@ -434,22 +438,15 @@ TextbufferDiary::select_searchstr_previous()
     if( iter_bound_start.is_start() )
         return false;
 
-    iter_bound_end = iter_bound_start;
-
-    if( ! iter_bound_start.backward_to_tag_toggle( m_tag_match ) )
-        return false;
-    iter_bound_start.backward_to_tag_toggle( m_tag_match );
-    iter_bound_start.backward_to_tag_toggle( m_tag_match );
-
-    if( ! iter_bound_end.backward_search( m_search_str,
-                                          Gtk::TextSearchFlags( 0 ),
-                                          iter_start,
-                                          iter_end,
-                                          iter_bound_start ) )
+    if( ! iter_bound_start.backward_search( m_search_str,
+                                            Gtk::TEXT_SEARCH_CASE_INSENSITIVE,
+                                            iter_start,
+                                            iter_end,
+                                            begin() ) )
         return false;
 
-    select_range( iter_start, iter_end );
     m_ptr2textview->scroll_to( iter_start );
+    select_range( iter_start, iter_end );
     return true;
 }
 
@@ -465,45 +462,35 @@ TextbufferDiary::select_searchstr_next()
     if( iter_bound_end.is_end() )
         return false;
 
-    iter_bound_start = iter_bound_end;
-
-    if( ! iter_bound_end.forward_to_tag_toggle( m_tag_match ) )
-        return false;
-    iter_bound_end.forward_to_tag_toggle( m_tag_match );
-    if( iter_bound_end.has_tag( m_tag_match ) )
-        iter_bound_end.forward_to_tag_toggle( m_tag_match );
-
-    if( ! iter_bound_start.forward_search( m_search_str,
-                                           Gtk::TextSearchFlags( 0 ),
-                                           iter_start,
-                                           iter_end,
-                                           iter_bound_end ) )
+    if( ! iter_bound_end.forward_search( m_search_str,
+                                         Gtk::TEXT_SEARCH_CASE_INSENSITIVE,
+                                         iter_start,
+                                         iter_end,
+                                         end() ) )
         return false;
 
-    select_range( iter_start, iter_end );
     m_ptr2textview->scroll_to( iter_start );
+    select_range( iter_start, iter_end );
     return true;
 }
 
 void
-TextbufferDiary::parse( const Gtk::TextIter& i_start, const Gtk::TextIter& i_end )
+TextbufferDiary::parse( int o_start, int o_end )
 {
-    int o_start( i_start.get_offset() ), o_end( i_end.get_offset() );
-
     m_flag_parsing = true;
     m_flag_ongoing_operation++;
 
     // COMPLETELY CLEAR THE PARSING REGION
     if( ! m_flag_settextoperation )
     {
-        remove_all_tags( i_start, i_end );
-
         if( ! m_flag_updating_search_text )
         {
             clear_images( o_start, o_end ); // alters the buffer
             clear_links( o_start, o_end );
         }
 
+        remove_all_tags( get_iter_at_offset( o_start ), get_iter_at_offset( o_end ) );
+
         m_ptr2entry->set_text( get_text() );
     }
 
@@ -514,6 +501,9 @@ TextbufferDiary::parse( const Gtk::TextIter& i_start, const Gtk::TextIter& i_end
 
     m_flag_ongoing_operation--;
     m_flag_parsing = false;
+
+    if( !Diary::d->is_read_only() )
+        reset_markup_visibilities();
 }
 
 Wchar
@@ -525,9 +515,8 @@ TextbufferDiary::get_char_at( int i )
 bool
 TextbufferDiary::check_cursor_is_in_para()
 {
-    unsigned int pos_cursor = get_insert()->get_iter().get_offset();
-    return( ( pos_cursor > m_parser_pos_cur_para_begin ) &&
-            ( pos_cursor <= m_parser_pos_cur_para_end ) );
+    return( ( m_pos_current > m_parser_pos_cur_para_begin ) &&
+            ( m_pos_current <= m_parser_pos_cur_para_end ) );
 }
 
 // PARSING APPLIERS
@@ -614,8 +603,8 @@ TextbufferDiary::apply_hidden_link_tags( Gtk::TextIter& iter_end,
     Gtk::TextIter iter_current( get_iter_at_offset( m_pos_current ) );
     Gtk::TextIter iter_label( get_iter_at_offset( pos_tab + 1 ) );
 
-    apply_tag( m_tag_markup_link, iter_start, iter_label );
-    apply_tag( m_tag_markup_link, iter_current, iter_end );
+    apply_tag( m_tag_markup, iter_start, iter_label );
+    apply_tag( m_tag_markup, iter_current, iter_end );
     remove_tag( m_tag_misspelled, iter_start, iter_label );
 
     apply_tag( tag_link, iter_label, iter_current );
@@ -634,11 +623,8 @@ TextbufferDiary::apply_link()
 {
     // HANDLE RELATIVE PATHS
     if( m_word_last.find( "rel://" ) == 0 )
-    {
         m_word_last.replace( 0, 5, "file://" + Glib::path_get_dirname( Diary::d->get_path() ) );
-    }
 
-    Gtk::TextIter iter_start( get_iter_at_offset( m_pos_start ) );
     Gtk::TextIter iter_current( get_iter_at_offset( m_pos_current ) );
 
     if( m_flag_hidden_link )
@@ -657,6 +643,10 @@ TextbufferDiary::apply_link()
     {
         PRINT_DEBUG( "url: " + m_word_last );
 
+        Gtk::TextIter iter_start( get_iter_at_offset( m_pos_start ) );
+        remove_tag( m_tag_misspelled, iter_start, iter_current );
+
+        // check if link is to an image file:
         if( m_word_last.find( "file:///" ) == 0  &&
             // there can be no text on the same line as the image:
             iter_start.starts_line() && iter_current.ends_line() )
@@ -665,43 +655,35 @@ TextbufferDiary::apply_link()
             {
                 Glib::RefPtr< Gdk::Pixbuf > buf(
                         Lifeograph::get_thumbnail( Glib::filename_from_uri( m_word_last ),
-                                                  m_max_thumbnail_width ) );
+                                                   m_max_thumbnail_width ) );
 
-                remove_tag( m_tag_misspelled, iter_start, iter_current );
+                iter_current = insert( iter_current, "\n" );
+                iter_current = insert_pixbuf( iter_current, buf );
+                iter_start = get_iter_at_offset( m_pos_start );
+
+                m_pos_current += 2;
+                m_pos_end += 2;
+
+                auto iter_img_start{ iter_current };
+                iter_img_start.backward_cursor_positions( 2 );
                 apply_tag( m_tag_link, iter_start, iter_current );
+                apply_tag( m_tag_markup, iter_start, iter_img_start );
+                apply_tag( m_tag_image, iter_img_start, iter_current );
 
-                //if( ( m_pos_current <= m_parser_pos_cur_para_begin ) ||
-                  //  ( m_pos_current > m_parser_pos_cur_para_end ) )
-                {
-                    apply_tag( m_tag_hidden, iter_start, iter_current );
-
-                    m_list_images.insert( new InlineImage( iter_current.get_offset() ) );
-                    insert_pixbuf( iter_current, buf );
-
-                    m_pos_current++;
-                    m_pos_end++;
-
-                    iter_start = get_iter_at_offset( m_pos_start );
-                    iter_current = get_iter_at_offset( m_pos_current );
-                    apply_tag( m_tag_image, iter_start, iter_current );
-                }
             }
             catch( Glib::FileError& er )
             {
                 print_error( "Link target not found" );
-                remove_tag( m_tag_misspelled, iter_start, iter_current );
                 apply_tag( m_tag_link_broken, iter_start, iter_current );
             }
             catch( Gdk::PixbufError& er )
             {
                 PRINT_DEBUG( "Link is not an image" );
-                remove_tag( m_tag_misspelled, iter_start, iter_current );
                 apply_tag( m_tag_link, iter_start, iter_current );
             }
         }
         else
         {
-            remove_tag( m_tag_misspelled, iter_start, iter_current );
             apply_tag( m_tag_link, iter_start, iter_current );
         }
 
@@ -843,10 +825,6 @@ TextbufferDiary::apply_indent()
     tag_indent->property_left_margin() = tab_width; // FIXME: indents more than expected
     tag_indent->property_indent() = -tab_width;
 
-    PRINT_DEBUG( "tab width= " , tab_width );
-    PRINT_DEBUG( "pos_current= " , m_pos_current );
-    PRINT_DEBUG( "m_pos_start= " , m_pos_start );
-
     get_tag_table()->add( tag_indent );
 
     Gtk::TextIter iter_start( get_iter_at_offset( m_pos_start ) );
@@ -859,21 +837,15 @@ TextbufferDiary::apply_indent()
 inline void
 TextbufferDiary::apply_markup( const Glib::RefPtr< Tag >& tag )
 {
-    bool flag_hide( !check_cursor_is_in_para() );
-
     Gtk::TextIter iter_start( get_iter_at_offset( m_pos_start ) );
     Gtk::TextIter iter_current( get_iter_at_offset( m_pos_current ) );
     Gtk::TextIter iter2( iter_start );
     iter2++;
     apply_tag( m_tag_markup, iter_start, iter2 );
-    if( flag_hide )
-        apply_tag( m_tag_hidden, iter_start, iter2 );
     apply_tag( tag, iter2, iter_current );
     iter2 = iter_current;
     iter2++;
     apply_tag( m_tag_markup, iter_current, iter2 );
-    if( flag_hide )
-        apply_tag( m_tag_hidden, iter_current, iter2 );
 }
 
 void
@@ -887,91 +859,6 @@ TextbufferDiary::update_todo_status()
 }
 
 // EVENT HANDLERS
-void
-TextbufferDiary::update_cursor_paragraph( const Gtk::TextIter& iterator )
-{
-    Gtk::TextIter iter_begin( iterator );
-    Gtk::TextIter iter_end( iter_begin );
-    calculate_para_bounds( iter_begin, iter_end );
-
-    m_parser_pos_cur_para_begin = iter_begin.get_offset();
-    m_parser_pos_cur_para_end = iter_end.get_offset();
-}
-
-void
-TextbufferDiary::clear_cursor_paragraph( const Gtk::TextIter& iterator )
-{
-    Gtk::TextIter iter_begin( iterator );
-    Gtk::TextIter iter_end( iterator );
-    calculate_para_bounds( iter_begin, iter_end );
-
-    remove_tag( m_tag_hidden, iter_begin, iter_end );
-    int i_begin = iter_begin.get_offset();
-    int i_end = iter_end.get_offset();
-    clear_images( i_begin, i_end );
-}
-
-void
-TextbufferDiary::on_insert( const Gtk::TextIter& iterator,
-                            const Glib::ustring& text,
-                            int bytes )
-{
-    PRINT_DEBUG( "TextbufferDiary::on_insert()" );
-
-    // REPLACE \r chars
-    std::string text2( text ); // to drop constness
-    std::replace( text2.begin(), text2.end(), '\r', '\n' );
-
-    const Glib::ustring::size_type offset_itr = iterator.get_offset();
-
-    // UNDO
-    if( UndoManager::m->is_freezed() )
-    {
-        Gtk::TextIter iter_scroll( iterator );  // to remove constness
-        m_ptr2textview->scroll_to( iter_scroll );
-    }
-    else
-    {
-        UndoInsert* undo_insert = new UndoInsert( offset_itr, text2 );
-        UndoManager::m->add_action( undo_insert );
-    }
-
-    if( text == "-" && handle_minus() )
-    {   // curly bracket is necessary for non-debug build
-        PRINT_DEBUG( "em dash inserted" );
-    }
-    else
-        Gtk::TextBuffer::on_insert( iterator, text2, bytes );
-
-    if( m_flag_ongoing_operation || m_flag_parsing )
-        return;
-
-    //  CALCULATING LIMITS & PARSING
-    if( m_flag_settextoperation )
-    {
-        parse( begin(), end() );
-    }
-    else
-    {
-        update_offsets( offset_itr, text2.length() );
-
-        Gtk::TextIter iter_begin( get_iter_at_offset( offset_itr ) );
-        Gtk::TextIter iter_end( get_iter_at_offset( offset_itr + text2.length() ) );
-        calculate_para_bounds( iter_begin, iter_end );
-
-        PRINT_DEBUG( "cursor: ", get_insert()->get_iter().get_offset() );
-        PRINT_DEBUG( "    para: (", m_parser_pos_cur_para_begin,
-                             ", ", m_parser_pos_cur_para_end, ")" );
-
-        PRINT_DEBUG( "parsed slice: \"\"\"", get_slice( iter_begin, iter_end ), "\"\"\"" );
-
-        // TODO: update cur paragraph
-
-        parse( iter_begin, iter_end );
-        m_flag_outstanding_change = true;
-    }
-}
-
 void
 TextbufferDiary::handle_space()
 {
@@ -1228,14 +1115,48 @@ TextbufferDiary::increment_numbered_line( Gtk::TextIter& iter, int expected_valu
     return false;
 }
 
+void
+TextbufferDiary::on_insert( const Gtk::TextIter& iterator,
+                            const Glib::ustring& text,
+                            int bytes )
+{
+    PRINT_DEBUG( "TextbufferDiary::on_insert()" );
+
+    // REPLACE \r chars
+    std::string text2( text ); // to drop constness
+    std::replace( text2.begin(), text2.end(), '\r', '\n' );
+
+    const Glib::ustring::size_type offset_itr = iterator.get_offset();
+
+    // UNDO
+    if( UndoManager::m->is_freezed() )
+    {
+        Gtk::TextIter iter_scroll( iterator );  // to remove constness
+        m_ptr2textview->scroll_to( iter_scroll );
+    }
+    else
+    {
+        UndoInsert* undo_insert = new UndoInsert( offset_itr, text2 );
+        UndoManager::m->add_action( undo_insert );
+    }
+
+    if( text == "-" && handle_minus() )
+    {   // curly bracket is necessary for non-debug build
+        PRINT_DEBUG( "em dash inserted" );
+    }
+    else
+        Gtk::TextBuffer::on_insert( iterator, text2, bytes );
+
+    m_flag_outstanding_change = true;
+}
+
 void
 TextbufferDiary::on_erase( const Gtk::TextIter& iter_start,
                            const Gtk::TextIter& iter_end )
 {
     PRINT_DEBUG( "TextbufferDiary::on_erase()" );
 
-    const Glib::ustring::size_type offset_itr = iter_start.get_offset();
-    const int delta = offset_itr - iter_end.get_offset();
+    const int offset_itr = iter_start.get_offset();
 
     if( UndoManager::m->is_freezed() )
     {
@@ -1250,81 +1171,53 @@ TextbufferDiary::on_erase( const Gtk::TextIter& iter_start,
 
     Gtk::TextBuffer::on_erase( iter_start, iter_end );
 
-    // set_text() calls on_erase too:
-    if( m_flag_settextoperation || m_flag_ongoing_operation || m_flag_parsing )
-        return;
-
-    update_offsets( offset_itr, delta );
-
-    //  CALCULATING LIMITS & PARSING
-    Gtk::TextIter iter_start2( get_iter_at_offset( offset_itr ) );
-    Gtk::TextIter iter_end2( iter_start2 );
-    calculate_para_bounds( iter_start2, iter_end2 );
-    parse( iter_start2, iter_end2 );
     m_flag_outstanding_change = true;
 }
 
-/*void
-TextbufferDiary::on_apply_tag( const Glib::RefPtr< TextBuffer::Tag >& tag,
-                               const Gtk::TextIter& iter_begin,
-                               const Gtk::TextIter& iter_end )
-{
-    // do not check spelling of links:
-    if( ( iter_begin.has_tag( m_tag_link ) || iter_begin.has_tag( m_tag_hidden ) ) &&
-        tag == m_tag_misspelled )
-        return;
-    else
-        Gtk::TextBuffer::on_apply_tag( tag, iter_begin, iter_end );
-}*/
-
 void
 TextbufferDiary::on_mark_set( const TextBuffer::iterator& iter,
                               const Glib::RefPtr< TextBuffer::Mark >& mark )
 {
-    if( m_ptr2textview && m_max_thumbnail_width > 0 )
+    if( m_ptr2textview && m_max_thumbnail_width > 0 && mark == get_insert() )
+    {
+        PRINT_DEBUG( "TextbufferDiary::on_mark_set()" );
         m_flag_outstanding_change = true;
+    }
     Gtk::TextBuffer::on_mark_set( iter, mark );
 }
 
 void
 TextbufferDiary::handle_event_after( GdkEvent* )
 {
-    if( m_flag_outstanding_change && ! m_flag_ongoing_operation && ! Diary::d->is_read_only() )
+    if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN &&
+        m_flag_outstanding_change && ! m_flag_ongoing_operation &&
+        ! Diary::d->is_read_only() )
     {
 #if LIFEOGRAPH_DEBUG_BUILD
-        PRINT_DEBUG( "******** TextbufferDiary::handle_event_after() ********" );
-        PRINT_DEBUG( "    para: (", m_parser_pos_cur_para_begin,
+        PRINT_DEBUG( "*** TextbufferDiary::handle_event_after() *** ",
+                     "para: (", m_parser_pos_cur_para_begin,
                      ", ", m_parser_pos_cur_para_end, ")" );
-
-        for( auto img : m_list_images )
-            PRINT_DEBUG( "img: ", img->m_offset );
 #endif
 
-        Glib::RefPtr< Gtk::TextMark > mark = get_insert();
+        // UPDATE PARA BOUNDARIES
+        Gtk::TextIter   iter_bgn    { get_insert()->get_iter() };
+        Gtk::TextIter   iter_end    { iter_bgn };
 
-        if( m_parser_pos_cur_para_begin != Wstring::npos )
-        {
-            if( ! check_cursor_is_in_para() )
-            {
-                // clear the current paragraph
-                clear_cursor_paragraph( mark->get_iter() );
+        m_pos_cursor = iter_bgn.get_offset();
 
-                // fix the previous paragraph
-                parse( get_iter_at_offset( m_parser_pos_cur_para_begin ),
-                       get_iter_at_offset( m_parser_pos_cur_para_end ) );
+        calculate_para_bounds( iter_bgn, iter_end );
 
-                // update the paragraph bounds
-                update_cursor_paragraph( mark->get_iter() );
-            }
-        }
-        else
-        {
-            // clear the current paragraph
-            clear_cursor_paragraph( mark->get_iter() );
+        if( iter_bgn.has_tag( m_tag_image ) )
+            iter_bgn.backward_to_tag_toggle( m_tag_link );
 
-            // update the paragraph bounds
-            update_cursor_paragraph( mark->get_iter() );
-        }
+        if( iter_end.has_tag( m_tag_image ) )
+            iter_end.forward_to_tag_toggle( m_tag_image );
+
+        m_parser_pos_cur_para_begin = iter_bgn.get_offset();
+        m_parser_pos_cur_para_end = iter_end.get_offset();
+
+        // PARSE
+        parse( m_parser_pos_cur_para_begin, m_parser_pos_cur_para_end );
 
         m_flag_outstanding_change = false;
     }
@@ -1387,51 +1280,25 @@ TextbufferDiary::clear_images( int start, int& end )
 {
     PRINT_DEBUG( "TextbufferDiary::clear_images( ", start, ", ", end, " )" );
 
-    //assert( m_flag_parsing );   // this function should only be called during parsing
-
     m_flag_ongoing_operation++;
 
-    for( ListImages::iterator iter = m_list_images.begin(); iter != m_list_images.end(); )
+    for( auto iter = get_iter_at_offset( start ); iter.forward_to_tag_toggle( m_tag_image ); )
     {
-        InlineImage* img( *iter );
+        if( iter.get_offset() > end )
+            break;
 
-        if( start <= img->m_offset && end >= img->m_offset )
-        {
-            // REMOVE IMAGE FROM THE LIST
-            m_list_images.erase( iter++ );
+        if( iter.starts_tag( m_tag_image ) )
+            iter.forward_to_tag_toggle( m_tag_image );
 
-            // REMOVE IMAGE FROM THE BUFFER
-            Gtk::TextIter titer_begin( get_iter_at_offset( img->m_offset ) );
-            Gtk::TextIter titer_end( titer_begin );
-            erase( titer_begin, ++titer_end );
-
-            // UPDATE OFFSETS
-            for( InlineImage* img2 : m_list_images )
-            {
-                if( img->m_offset < img2->m_offset )
-                    img2->m_offset--;
-            }
-
-            delete( img );
-            end--;
-        }
-        else
-            ++iter;
+        auto iter_begin{ iter };
+        iter_begin.backward_cursor_positions( 2 );
+        iter = erase( iter_begin, iter );
+        end--;
     }
 
     m_flag_ongoing_operation--;
 }
 
-void
-TextbufferDiary::clear_images() // note that this version does not alter the buffer
-{
-    for( InlineImage* img : m_list_images )
-    {
-        delete( img );
-    }
-    m_list_images.clear();
-}
-
 bool
 TextbufferDiary::update_thumbnail_width( int width )
 {
@@ -1454,22 +1321,25 @@ TextbufferDiary::update_thumbnail_width( int width )
 }
 
 void
-TextbufferDiary::update_offsets( int pos, int delta )
+TextbufferDiary::reset_markup_visibilities()
 {
-    PRINT_DEBUG( "TextbufferDiary::update_image_offsets( ", pos, ", ", delta, " )" );
-    PRINT_DEBUG( "para - before: (", m_parser_pos_cur_para_begin, ", ", m_parser_pos_cur_para_end, ")" );
+    auto iter{ begin() };
+    auto iter_end{ iter };
+    int offset;
 
-    for( InlineImage* img : m_list_images )
+    while( iter.forward_to_tag_toggle( m_tag_markup ) )
     {
-        if( pos < img->m_offset )
-            img->m_offset += delta;
+        iter_end = iter;
+        iter_end.forward_to_tag_toggle( m_tag_markup );
+        offset = iter.get_offset();
+
+        if( ! iter.has_tag( m_tag_hidden ) &&
+            ( offset < m_parser_pos_cur_para_begin ||
+              offset > m_parser_pos_cur_para_end ) )
+            apply_tag( m_tag_hidden, iter, iter_end );
+
+        iter = iter_end;
     }
-
-    if( ( unsigned ) pos < m_parser_pos_cur_para_begin )
-        m_parser_pos_cur_para_begin += delta;
-
-    if( ( unsigned ) pos <= m_parser_pos_cur_para_end )
-        m_parser_pos_cur_para_end += delta;
 }
 
 void
@@ -1639,13 +1509,15 @@ TextbufferDiary::toggle_format( Glib::RefPtr< Tag > tag, const Glib::ustring& ma
             if( iter_start.starts_word() )
                 iter_start++;
 
-            int offset_start( iter_start.get_line_offset() );
+            iter_end = iter_start;
 
             iter_start.backward_to_tag_toggle( tag );
+            iter_end.forward_to_tag_toggle( TextbufferDiary::m_tag_markup );
+            int offset_end( iter_end.get_offset() );
             backspace( iter_start );
 
-            iter_end = get_iter_at_offset( offset_start );
-            iter_end.forward_to_tag_toggle( TextbufferDiary::m_tag_markup );
+            iter_end = get_iter_at_offset( offset_end );
+            iter_end--;
 
             m_flag_ongoing_operation--;
 
@@ -1713,6 +1585,7 @@ TextbufferDiary::set_comment_visibility( bool visible )
 void
 TextbufferDiary::calculate_para_bounds( Gtk::TextIter& iter_begin, Gtk::TextIter& iter_end )
 {
+    PRINT_DEBUG( "TextbufferDiary::calculate_para_bounds()" );
     if( ! iter_begin.backward_find_char( s_predicate_nl ) )
         iter_begin.set_offset( 0 );
 
@@ -2122,9 +1995,8 @@ TextbufferDiary::set_richtext( Entry* entry )
 {
     m_flag_settextoperation = true;
     clear_links();
-    clear_images();
-    m_parser_pos_cur_para_begin = Wstring::npos;
-    m_parser_pos_cur_para_end = Wstring::npos;
+    m_parser_pos_cur_para_begin = -1;
+    m_parser_pos_cur_para_end = -1;
     UndoManager::m->clear();
     UndoManager::m->freeze();
     m_ptr2entry = entry;
@@ -2134,6 +2006,7 @@ TextbufferDiary::set_richtext( Entry* entry )
     Gtk::TextBuffer::set_text( entry->get_text() );
     place_cursor( begin() );
     UndoManager::m->thaw();
+    reparse();
     m_flag_settextoperation = false;
 }
 
@@ -2207,7 +2080,6 @@ TextbufferDiary::set_theme( const Theme* theme )
     m_tag_match->property_background_rgba() = contrast2(
             theme->color_base, Theme::s_color_match1, Theme::s_color_match2 );
     m_tag_markup->property_foreground_rgba() = color_mid;
-    m_tag_markup_link->property_foreground_rgba() = color_mid;
     m_tag_link->property_foreground_rgba() = contrast2(
             theme->color_base, Theme::s_color_link1, Theme::s_color_link2 );
     m_tag_link_broken->property_foreground_rgba() = contrast2(
@@ -2416,12 +2288,12 @@ TextbufferDiary::handle_language_changed( const std::string& lang,
             if( set_language( Diary::d->get_lang() ).empty() )
                 remove_tag( m_tag_misspelled, begin(), end() );
             else
-                parse( begin(), end() );
+                reparse();
         }
         else
         {
             m_ptr2entry->set_lang( set_language( lang ) );
-            parse( begin(), end() );
+            reparse();
         }
     }
 }
@@ -2430,7 +2302,7 @@ void
 TextbufferDiary::ignore_misspelled_word( const Glib::ustring& word )
 {
     enchant_dict_add_to_session( m_enchant_dict, word.c_str(), word.size() );
-    parse( begin(), end() );   // check entire buffer again
+    reparse();   // check entire buffer again
 }
 
 void
@@ -2473,7 +2345,7 @@ void
 TextbufferDiary::add_word_to_dictionary( const Glib::ustring& word )
 {
     enchant_dict_add( m_enchant_dict, word.c_str(), word.size() );
-    parse( begin(), end() );   // check entire buffer again
+    reparse();   // check entire buffer again
 }
 
 // TEXTVIEW ========================================================================================
