@@ -28,7 +28,7 @@
 
 extern "C"
 {
-#include <enchant-2/enchant.h>
+#include <enchant.h>
 }
 
 #include "helpers.hpp"
@@ -90,11 +90,7 @@ class UndoEdit : public Undoable
             m_ptr2buffer->erase( iter_start, iter_end );
         }
 
-        void                insert()
-        {
-            Gtk::TextIter iterator = m_ptr2buffer->get_iter_at_offset( m_position );
-            m_ptr2buffer->insert( iterator, m_text );
-        }
+        void                insert();
 
         unsigned int        m_position;
         Glib::ustring       m_text;
@@ -264,15 +260,6 @@ class LinkCheck : public Link
         unsigned int                m_state_index;
 };
 
-class InlineImage
-{
-    public:
-        InlineImage( int );
-        ~InlineImage();
-
-        int                         m_offset;
-};
-
 // NEW LINE PREDICATE ==============================================================================
 class PredicateNL
 {
@@ -353,13 +340,6 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         void                        on_erase( const Gtk::TextIter&,
                                               const Gtk::TextIter& );
 
-        /*void                      on_apply_tag( const Glib::RefPtr< TextBuffer::Tag >&,
-                                                  const Gtk::TextIter&,
-                                                  const Gtk::TextIter& );*/
-
-        void                        update_cursor_paragraph( const Gtk::TextIter& );
-        void                        clear_cursor_paragraph( const Gtk::TextIter& );
-
         void                        on_mark_set( const TextBuffer::iterator&,
                                                  const Glib::RefPtr< TextBuffer::Mark >& );
 
@@ -369,8 +349,8 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         bool                        increment_numbered_line( Gtk::TextIter&, int );
 
         // PARSING
-        void                        parse( const Gtk::TextIter&, const Gtk::TextIter& );
-        void                        reparse() { parse( begin(), end() ); }
+        void                        parse( int, int );
+        void                        reparse() { parse( 0, end().get_offset() ); }
 
         void                        apply_hidden_link_tags( Gtk::TextIter&,
                                             const Glib::RefPtr< Tag >& );
@@ -418,8 +398,9 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         void                        replace_misspelled_word( const Glib::ustring& );
         void                        add_word_to_dictionary( const Glib::ustring& );
 
-        Wstring::size_type          m_parser_pos_cur_para_begin{ 0 };
-        Wstring::size_type          m_parser_pos_cur_para_end{ 0 };
+        int                         m_parser_pos_cur_para_begin{ 0 };
+        int                         m_parser_pos_cur_para_end{ 0 };
+        int                         m_pos_cursor{ 0 };
 
         static PredicateNL          s_predicate_nl;
 
@@ -430,7 +411,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         void                        clear_images( int, int& );
         void                        clear_images();
         bool                        update_thumbnail_width( int );
-        void                        update_offsets( int, int );
+        void                        reset_markup_visibilities();
 
         // TAGS
         // TODO: move these program-wide objects out of this class as...
@@ -439,7 +420,6 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         Glib::RefPtr< Tag >         m_tag_subheading;
         Glib::RefPtr< Tag >         m_tag_match;
         Glib::RefPtr< Tag >         m_tag_markup;
-        Glib::RefPtr< Tag >         m_tag_markup_link;
         Glib::RefPtr< Tag >         m_tag_hidden;
         Glib::RefPtr< Tag >         m_tag_bold;
         Glib::RefPtr< Tag >         m_tag_italic;
@@ -472,9 +452,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
 
         typedef std::list< Link* >  ListLinks;
         ListLinks                   m_list_links;
-        typedef std::set< InlineImage* >
-                                    ListImages;
-        ListImages                  m_list_images;
+
         int                         m_max_thumbnail_width{ 0 };
 
         // ENCHANT SPELLCHECKER
@@ -484,6 +462,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
     friend class EntryView;
     friend class LinkCheck;
     friend class TextviewDiary;
+    friend class UndoEdit;
 };
 
 // TEXTVIEW ========================================================================================
